//
// Copyright 2016-2021 by Garmin Ltd. or its subsidiaries.
// Subject to Garmin SDK License Agreement and Wearables
// Application Developer Agreement.
//

import Toybox.Application.Storage;
import Toybox.Lang;
import Toybox.WatchUi;

var tz1;
var tz2;
var ampm;
var ccw;
var secm;
var sec_idx;
var backing;
var tz_cache;

var ampm_label as Array<String> = [
    "AM/PM off",
    "PM only",
    "AM and PM"
];

var sec_label as Array<String> = [
    "Sweep hand",
    "Tick marks",
    "Diamond"
];

var sec_colors as Array<String> = [
    "Red",
    "Green",
    "Blue",
    "Cyan",
    "Magenta",
    "Yellow",
    "Custom"
];

var zone12;
var zone_old;
var zone_name;

var tz1_menu;
var tz2_menu;

var menu2 as Menu2 or Null;     // note the comment on the onBack() routine
var cache as TzCache or Null;

//! The app settings menu
class Menu extends WatchUi.Menu2 {

//! Constructor
public function initialize()
{

    C.settings();

    tz1 = C.tz1;
    tz2 = C.tz2;
    ampm = C.ampm;
    ccw = C.ccw;
    secm = C.secm;
    sec_idx = C.sec_idx;
    backing = C.backing;
    tz_cache = false;

C.pr_mem("menu init");
    Menu2.initialize(null);
    Menu2.setTitle("Config:" + V.version);

    tz1_menu = new MenuItem("Timezone 1", C.tz1_info[4], C.MENU_TZ1, {}); 
    Menu2.addItem(tz1_menu);

    tz2_menu = new MenuItem("Timezone 2", C.tz2_info[4], C.MENU_TZ2, {}); 
    Menu2.addItem(tz2_menu);

    Menu2.addItem(new MenuItem("Seconds", null, C.MENU_SECOND, {}));

    Menu2.addItem(new MenuItem("Controls", null, C.MENU_CONTROL, {}));
C.pr_mem("!menu init");
}

}

//! Input handler for the app settings menu
class MenuDelegate extends WatchUi.Menu2InputDelegate
{

public function initialize()
{

    Menu2InputDelegate.initialize();
}

public function onSelect(menuItem as MenuItem)
{

    var id = (menuItem.getId() as Number).toNumber();
    //C.logger("onSelect:" + id);
    var t_menu = menuItem as ToggleMenuItem;
    if (id == C.MENU_TZ1) {
        menu_tz("Timezone 1", C.MENU_TZ1, C.tz1_info);
    } else if (id == C.MENU_TZ2) {
        menu_tz("Timezone 2", C.MENU_TZ2, C.tz2_info);
    } else if (id == C.MENU_SECOND) {
        sec_menu();
    } else if (id == C.MENU_CONTROL) {
        ctl_menu();
    } else if (id == C.MENU_SHAND) {
        secm += 1;
        if (secm > 2) {
            secm = 0; }
        menuItem.setSubLabel(sec_label[secm]);
    } else if (id == C.MENU_SCOLOR) {
        sec_idx += 1;
        if (sec_idx > 6) {
            sec_idx = 0; }
        menuItem.setSubLabel(sec_colors[sec_idx]);
    } else if (id == C.MENU_AMPM) {
        ampm += 1;
        if (ampm > 2) {
            ampm = 0; }
        menuItem.setSubLabel(ampm_label[ampm]);
    } else if (id == C.MENU_CCW) {
        ccw = t_menu.isEnabled() ? -1 : 1;
    } else if (id == C.MENU_BACKING) {
        backing = t_menu.isEnabled();
    } else if (id == C.MENU_CACHE) {
        tz_cache = t_menu.isEnabled();
    } else if (id == C.MENU_TZ_CACHE) {
        menu_set_cache();
    } else if (id == C.MENU_TZ_USA) {
        menu_set_tzs("USA/LOCAL", 0, TzOffsets.off[0]);
    } else if (id >= C.MENU_TZ_A && id <= C.MENU_TZ_Z) {
        var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        var idx = id - C.MENU_TZ_A;
        menu_set_tzs(letters.substring(idx, idx+1), TzOffsets.off[idx], TzOffsets.off[idx+1]);
    } else if (id >= C.MENU_ZONES) {
        toggle(id, menuItem as ToggleMenuItem);
    }
}

//
//  Turns out this routine has implications for the global variable
//  menu2.  Normally, menu2 is needed to update items in the current
//  menu.  The current flow is that going to submenu A will set menu2
//  to A's info.  Then going to submenu B will set menu2 to B's info.
//  onBack() will then return to submenu A's context while leaving
//  menu2 with B's info.  This is currently not a problem since only
//  leaf contexts actually change the menu2 info.  If the code ever
//  changes to have non-leaf menu context change the menu2 info then
//  this will have to be redesigned.
//
public function onBack()
{

    //C.logger("onBack()");

    C.reconfig(tz1, tz2, ampm, ccw, secm, sec_idx, backing, tz_cache);
    tz_cache = false;

    WatchUi.popView(WatchUi.SLIDE_IMMEDIATE);

    return;
}

private function toggle(id, menu as ToggleMenuItem)
{

    //C.logger("toggle:" + zone_old + " -> " + id);
    if (zone_old != null) {
        var i = menu2.findItemById(zone_old);
        var m = menu2.getItem(i) as ToggleMenuItem;
        m.setEnabled(false);
    }
    zone_old = id;
    zone_name = menu.getLabel();
    menu.setEnabled(true);
    id -= C.MENU_ZONES;
    if (zone12 == C.MENU_TZ1) {
        tz1_menu.setSubLabel(menu.getLabel());
        tz1 = id;
    } else {
        tz2_menu.setSubLabel(menu.getLabel());
        tz2 = id;
    }
}

private function tz_cache_item(idx, active as String)
{

    var entry = cache.get(idx);
    var tz = entry[0];
    var enable = tz.equals(active) ? true : false;
    menu2.addItem(new ToggleMenuItem(tz, null, C.MENU_ZONES + entry[1], enable, {}));
    if (enable) {
        zone_old = C.MENU_ZONES + entry[1];
        zone_name = tz;
    }
}

private function menu_set_cache()
{

    menu2 = new Menu2({:title => "Cache entries"});
    cache = new TzCache();

    zone_old = null;

    for (var i = 0; i < cache.max; ++i) {
        tz_cache_item(i, zone_name); }

    WatchUi.pushView(menu2, new MenuDelegate(), WatchUi.SLIDE_UP);
}

private function menu_tz(title, idx, tz_info as Array<String or Number>)
{
    var letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";

    menu2 = new Menu2({:title => title});
    cache = new TzCache();

    zone12 = idx;
    zone_name = tz_info[4];

    menu2.addItem(new MenuItem("Cache", null, C.MENU_TZ_CACHE, {}));
    menu2.addItem(new MenuItem("USA/Local", null, C.MENU_TZ_USA, {}));
    for (var i = 0; i < 26; ++i) {
        if (TzOffsets.off[i] != TzOffsets.off[i + 1]) {
        menu2.addItem(new MenuItem(letters.substring(i,i+1), null, C.MENU_TZ_A + i, {})); }
    }

    WatchUi.pushView(menu2, new MenuDelegate(), WatchUi.SLIDE_UP);
}

private function tz_item(idx, name, active)
{

    var enable = name.equals(active) ? true : false;
    menu2.addItem(new ToggleMenuItem(name, null, C.MENU_ZONES + idx, enable, {}));
    if (enable) {
        zone_old = C.MENU_ZONES + idx;
        zone_name = name;
    }
}

private function menu_set_tzs(title, min, max)
{

    menu2 = new Menu2({:title => title});

    zone_old = null;

    for (var i = min; i < max; ++i) {
        tz_item(i, C.tz_get(i, -1)[4], zone_name); }

    WatchUi.pushView(menu2, new MenuDelegate(), WatchUi.SLIDE_UP);
}

private function sec_menu()
{

    menu2 = new Menu2({:title => "Second indicator"});

    menu2.addItem(new MenuItem("Seconds", sec_label[secm], C.MENU_SHAND, {})); 
    menu2.addItem(new MenuItem("Color", sec_colors[sec_idx], C.MENU_SCOLOR, {})); 

    WatchUi.pushView(menu2, new MenuDelegate(), WatchUi.SLIDE_UP);
}

private function ctl_menu()
{

    menu2 = new Menu2({:title => "Controls"});

    menu2.addItem(new MenuItem("AM/PM", ampm_label[ampm], C.MENU_AMPM, {})); 

    menu2.addItem(new ToggleMenuItem("Hands CCW", null, C.MENU_CCW, (ccw == -1), {})); 

    menu2.addItem(new ToggleMenuItem("Backing store", null, C.MENU_BACKING, backing, {})); 

    menu2.addItem(new ToggleMenuItem("Reset TZ cache", null, C.MENU_CACHE, tz_cache, {})); 

    WatchUi.pushView(menu2, new MenuDelegate(), WatchUi.SLIDE_UP);
}
    
}
